__all__ = ("ApiError", "NotFound", "ChapterVerse", "Bible")


class ApiError(Exception):
    def __init__(self, status: int, msg: str) -> None:
        super().__init__(f"Api has an error, return code: {status}.\n{msg}")


class NotFound(Exception):
    def __init__(self, book: str, chapter: int, verse: str) -> None:
        super().__init__(
            f"Book {book}, Chapter {chapter}, Verse(s) {verse} Wasn't Found."
        )


def _build_verse(start: int, end: int = None) -> str:
    return f"{start}{f'-{end}' if end else ''}"


def _build_citation(book: str, chapter: int, verse: str) -> str:
    return f"{book} {chapter}:{verse}"


class ChapterVerse:
    def __init__(self, verse: dict) -> None:
        del verse["book_id"]

        for key, val in verse.items():
            setattr(self, key, val)

    def __str__(self) -> str:
        return self.text

    @property
    def citation(self) -> str:
        return _build_citation(self.book_name, self.chapter, self.verse)


class Bible:
    def __init__(self, book: str) -> None:
        self.book = book

    @classmethod
    def request(
        cls, book: str, *, chapter: int, starting_verse: int, ending_verse: int = None
    ):
        try:
            import requests
        except ImportError:
            raise ImportError(
                "Please Install the requests module if you want to make a sync request."
            )

        self = cls(book)
        verse = _build_verse(starting_verse, ending_verse)

        self.request = requests.get(f"https://bible-api.com/{book}+{chapter}:{verse}")
        if self.request.status_code == 404:
            raise NotFound(book, chapter, verse)
        elif self.request.status_code > 202:
            raise ApiError(
                self.request.status_code, self.request.json().get("error", "")
            )

        self.json = self.request.json()
        self.verses = [ChapterVerse(i) for i in self.json["verses"]]

        return self

    @classmethod
    async def async_request(
        cls,
        book: str,
        *,
        chapter: int,
        starting_verse: int,
        ending_verse: int = None,
        loop=None,
    ):
        try:
            import aiohttp
        except ImportError:
            raise ImportError(
                "Please Install the aiohttp module if you want to make an async request."
            )

        self = cls(book)
        verse = _build_verse(starting_verse, ending_verse)

        async with aiohttp.ClientSession(loop=loop) as session:
            async with session.get(
                f"https://bible-api.com/{book}+{chapter}:{verse}"
            ) as resp:
                self.request = resp
                self.json = await resp.json()

        self.verses = [ChapterVerse(i) for i in self.json["verses"]]
        self.raw_verse = self.json["text"]
        if self.request.status == 404:
            raise NotFound(book, chapter, verse)
        elif self.request.status > 202:
            raise ApiError(self.request.status, self.json.get("error", ""))

        return self

    @property
    def citation(self) -> str:
        return self.json["reference"]

    @property
    def translation(self) -> str:
        return self.json["translation_name"]


from typing import Union, Optional

__all__ = (
    "Surah",
    "Ayah",
    "Search",
    "ApiError",
    "ContentTypeError",
    "NumberError",
    "WrongLang",
)


class ApiError(Exception):
    def __init__(self, status: int, msg: str) -> None:
        super().__init__(f"Api has an error, return code: {status}.\n{msg}")


class ContentTypeError(Exception):
    def __init__(
        self,
        class_: str,
        mode: str,
        first_query: Optional[Union[str, int]],
        second_query: Optional[Union[str, int]],
    ) -> None:
        super().__init__(
            f"Attempt to decode JSON with unexpected mimetype: Return code: None\nlink: http://api.alquran.cloud/v1/{mode}/{first_query}:{second_query}/en.asad"
            if class_ == "Ayah"
            else f"Attempt to decode JSON with unexpected mimetype: Return code: None\nlink: http://api.alquran.cloud/v1/{mode}/{first_query}/{second_query}/en.asad"
        )


class NumberError(Exception):
    def __init__(
        self,
        mode: int,
        obj: str,
        first_query: int,
        second_query: Optional[Union[str, int]],
    ) -> None:
        super().__init__(
            f"{obj} must above {first_query}"
            if mode == 0
            else f"{obj} must be between {first_query} to {second_query}"
        )


class WrongLang(Exception):
    def __init__(self, lang: str) -> None:
        super().__init__(
            f"The lang '{lang}' is not supported, it only support arabic(ar) and english(eng)"
        )


class Surah:
    def __init__(self, surah: int = None):
        self.surah = surah

    @classmethod
    def request(cls, surah: int = None):
        try:
            import requests
        except ImportError:
            raise ImportError(
                "Please Install the requests module if you want to make a sync request."
            )

        self = cls(surah)

        self.request = requests.get(
            f"https://api.alquran.cloud/v1/surah/{surah}/en.asad"
        ).json()
        self.data = self.request["data"]
        self.ayah = self.data["ayahs"]
        if self.request["code"] > 202:
            raise ApiError(code=self.request["code"], msg=self.request["data"])

        return self

    @classmethod
    async def async_request(cls, surah: int = None, *, loop=None):
        try:
            import aiohttp
        except ImportError:
            raise ImportError(
                "Please Install the aiohttp module if you want to make an async request."
            )

        self = cls(surah)
        async with aiohttp.ClientSession(loop=loop) as session:
            async with session.get(
                f"https://api.alquran.cloud/v1/surah/{surah}/en.asad"
            ) as resp:
                self.request = await resp.json()
        self.data = self.request["data"]
        self.ayah = self.data["ayahs"]
        if self.request["code"] > 202:
            raise ApiError(code=self.request["code"], msg=self.request["data"])
        return self

    @property
    def api_code(self):
        return self.request["code"]

    @property
    def api_status(self):
        return self.request["status"]

    def name(self, lang: str = "ar"):
        if lang == "ar" or lang.lower() == "arabic":
            return self.data["name"]

        elif lang == "eng" or lang.lower() == "english":
            return self.data["englishName"]

        else:
            raise WrongLang(lang=lang)

    @property
    def name_mean(self):
        return self.data["englishNameTranslation"]

    @property
    def revelation_type(self):
        return self.data["revelationType"]

    @property
    def number_ayahs(self):
        return self.data["numberOfAyahs"]

    def request_ayahs(self):
        data = []
        for number in range(Surah.request(self.surah).number_ayahs):
            data.append(f"{self.ayah[number]['text']}")
        return data

    async def ayah_info(
        self,
        ayah: int = 1,
        *,
        text: bool = True,
        number_in_quran: bool = True,
        number_in_surah: bool = True,
        juz: bool = True,
        manzil: bool = True,
        page: bool = True,
        ruku: bool = True,
        hizbquarter: bool = True,
        sajda: bool = True,
    ):
        if ayah <= 0:
            raise NumberError(mode=0, obj="ayah", first_query=1, second_query=None)

        elif ayah > int(Surah.request(self.surah).number_ayahs):
            raise NumberError(
                mode=1,
                obj="ayah",
                first_query=1,
                second_query=Surah.request(self.surah).number_ayahs,
            )

        else:
            ayah -= 1
            data = {
                "text": self.ayah[ayah]["text"] if text else None,
                "number in quran": self.ayah[ayah]["number"]
                if number_in_quran
                else None,
                "number in surah": self.ayah[ayah]["numberInSurah"]
                if number_in_surah
                else None,
                "juz": self.ayah[ayah]["juz"] if juz else None,
                "manzil": self.ayah[ayah]["manzil"] if manzil else None,
                "page": self.ayah[ayah]["page"] if text else None,
                "ruku": self.ayah[ayah]["ruku"] if number_in_quran else None,
                "hizbquarter": self.ayah[ayah]["hizbQuarter"]
                if number_in_surah
                else None,
                "sajda": self.ayah[ayah]["sajda"] if juz else None,
            }
            return data


class Ayah:
    def __init__(self, surah: int = None, *, ayah: int = None):
        self.surah = surah
        self.ayah = ayah

    @classmethod
    def request(cls, surah: int = None, *, ayah: int = None, loop=None):
        try:
            import requests
        except ImportError:
            raise ImportError(
                "Please Install the requests module if you want to make a sync request."
            )

        self = cls(surah, ayah=ayah)
        self.request = requests.get(
            f"http://api.alquran.cloud/v1/ayah/{surah}:{ayah}/en.asad"
        ).json()
        self.data = self.request["data"]
        if self.request["code"] > 202:
            raise ApiError(code=self.request["code"], msg=self.request["data"])
        return self

    @classmethod
    async def async_request(cls, surah: int = None, *, ayah: int = None, loop=None):
        try:
            import aiohttp
        except ImportError:
            raise ImportError(
                "Please Install the aiohttp module if you want to make an async request."
            )

        self = cls(surah, ayah=ayah)

        try:
            async with aiohttp.ClientSession(loop=loop) as session:
                async with session.get(
                    f"http://api.alquran.cloud/v1/ayah/{surah}:{ayah}/en.asad"
                ) as resp:
                    self.request = await resp.json()
        except aiohttp.client_exceptions.ContentTypeError:
            raise ContentTypeError(
                class_="Ayah", mode="ayah", first_query=surah, second_query=surah
            )

        self.data = self.request["data"]
        if self.request["code"] > 202:
            raise ApiError(code=self.request["code"], msg=self.request["data"])
        return self

    @property
    def api_code(self):
        return self.request["code"]

    @property
    def api_status(self):
        return self.request["status"]


class Search:
    def __init__(
        self,
        mention: str = None,
        *,
        surah: Optional[Union[str, int]] = None,
        request: int = None,
    ):
        self.mention = mention
        self.surah = surah
        self.req = request

    @classmethod
    async def async_request(
        cls,
        mention: str = None,
        *,
        surah: Optional[Union[str, int]] = None,
        request: int = None,
        loop=None,
    ):
        try:
            import aiohttp
        except ImportError:
            raise ImportError(
                "Please Install the aiohttp module if you want to make an async request."
            )

        self = cls(mention, surah=surah, request=request)

        try:
            async with aiohttp.ClientSession(loop=loop) as session:
                async with session.get(
                    f"http://api.alquran.cloud/v1/search/{mention}/{surah}/en.pickthall"
                ) as resp:
                    self.request = await resp.json()
        except aiohttp.client_exceptions.ContentTypeError:
            raise ContentTypeError(
                class_="Search", mode="search", first_query=mention, second_query=surah
            )

        self.data = self.request["data"]
        self.matches = self.data["matches"]
        if self.request["code"] > 202:
            raise ApiError(code=self.request["code"], msg=self.request["data"])

        return self

    @classmethod
    def request(
        cls,
        mention: str = None,
        *,
        surah: Optional[Union[str, int]] = None,
        request: int = None,
    ):
        try:
            import requests
        except ImportError:
            raise ImportError(
                "Please Install the requests module if you want to make a sync request."
            )

        self = cls(mention, surah=surah, request=request)
        self.request = requests.get(
            f"http://api.alquran.cloud/v1/search/{mention}/{surah}/en.pickthall"
        ).json()
        self.data = self.request["data"]
        self.matches = self.data["matches"]
        if self.request["code"] > 202:
            raise ApiError(code=self.request["code"], msg=self.request["data"])
        return self

    @property
    def count(self):
        return self.data["count"]

    @property
    def api_code(self):
        return self.request["code"]

    @property
    def api_status(self):
        return self.request["status"]

    def find(self):
        data = []
        if self.req == None:
            for num in range(self.data["count"]):
                data.append(self.matches[num]["text"])
            return data

        else:
            for num in range(self.req):
                data.append(self.matches[num]["text"])
            return data


from .bible import Bible


__all__ = ("ApiError", "NotFound", "Torah", "BibleOnly")


class ApiError(Exception):
    def __init__(self, status: int, msg: str) -> None:
        super().__init__(f"Api has an error, return code: {status}.\n{msg}")


class NotFound(Exception):
    def __init__(self, book: str, chapter: int, verse: str) -> None:
        super().__init__(
            f"Book {book}, Chapter {chapter}, Verse(s) {verse} Wasn't Found."
        )


class BibleOnly(Exception):
    def __init__(self, book: str) -> None:
        super().__init__(
            f"The book {book} wasn't found because its available only in Bible"
        )


class Torah(Bible):
    def __init__(self, book: str) -> None:
        super().__init__(book)

    @classmethod
    def request(
        cls, book: str, *, chapter: int, starting_verse: int, ending_verse: int = None
    ):
        if book.lower() not in [
            "genesis",
            "exodus",
            "leviticus",
            "numbers",
            "deuteronomy",
        ]:
            raise BibleOnly(book)

        self = super(Torah, cls).request(
            book,
            chapter=chapter,
            starting_verse=starting_verse,
            ending_verse=ending_verse,
        )

        return self

    @classmethod
    async def async_request(
        cls,
        book: str,
        *,
        chapter: int,
        starting_verse: int,
        ending_verse: int = None,
        loop=None,
    ):
        if book.lower() not in [
            "genesis",
            "exodus",
            "leviticus",
            "numbers",
            "deuteronomy",
        ]:
            raise BibleOnly(book)

        self = await super(Torah, cls).async_request(
            book,
            chapter=chapter,
            starting_verse=starting_verse,
            ending_verse=ending_verse,
            loop=loop,
        )

        return self

    @property
    def citation(self) -> str:
        return self.json["reference"]

    @property
    def translation(self) -> str:
        return self.json["translation_name"]


from setuptools import setup, find_packages
 
with open('README.md', 'r', encoding = 'utf-8') as f:
    long_description = f.read()

classifiers = [
  'Development Status :: 5 - Production/Stable',
  'Intended Audience :: Education',
  'Operating System :: Microsoft :: Windows :: Windows 10',
  'License :: OSI Approved :: MIT License',
  'Programming Language :: Python :: 3'
]
 
setup(
	name='holybooks',
	version='0.1.2',
	description='An Api Wrapper for extracting info from: quran api and bible api. I didn\'t made the document for this so for now read the file in my github(https://github.com/TheGenocides/holybooks) to find all functions there. I will add more holybooks in the future!',
	long_description=long_description,
	long_description_content_type = 'text/markdown',  
	author='TheGenocide',
	author_email='luke.genesis.hyder@gmail.com',
	license='MIT', 
	classifiers=classifiers,
	keywords=['holybooks', 'holybook', 'scripture', 'kitab', 'quran', 'bible', 'torah'], 
	packages=find_packages(),
	install_requires=['requests', 'aiohttp'] 
)